"""Command for asking questions about the codebase using RAG."""

import logging
import os
import uuid
from pathlib import Path
from typing import Any, TypedDict

from litellm import completion

from codemap.db.client import DatabaseClient
from codemap.llm import create_client
from codemap.processor.pipeline import ProcessingPipeline
from codemap.utils.cli_utils import loading_spinner, progress_indicator
from codemap.utils.config_loader import ConfigLoader

from .formatter import format_content_for_context
from .prompts import SYSTEM_PROMPT

logger = logging.getLogger(__name__)

# Default values for configuration
DEFAULT_MAX_CONTEXT_LENGTH = 8000
DEFAULT_MAX_CONTEXT_RESULTS = 10


class AskResult(TypedDict):
	"""Structured result for the ask command."""

	answer: str | None
	context: list[dict[str, Any]]


class AskCommand:
	"""
	Handles the logic for the `codemap ask` command.

	Interacts with the ProcessingPipeline, DatabaseClient, and an LLM to
	answer questions about the codebase using RAG. Maintains conversation
	history for interactive sessions.

	"""

	def __init__(
		self,
		repo_path: Path | None = None,
		model: str | None = None,
		api_key: str | None = None,
		api_base: str | None = None,
		config_loader: ConfigLoader | None = None,
	) -> None:
		"""
		Initializes the AskCommand, setting up clients and pipeline.

		Args:
		        repo_path: Path to the repository to analyze
		        model: LLM model to use for answering questions
		        api_key: API key for LLM provider
		        api_base: API base URL for LLM provider
		        config_loader: ConfigLoader to use for configuration (follows DI pattern)

		"""
		self.repo_path = repo_path or Path.cwd()
		self.session_id = str(uuid.uuid4())  # Unique session ID for DB logging

		# Accept ConfigLoader from higher-level components
		# Always ensure we have a valid ConfigLoader instance
		if config_loader is None:
			self.config_loader = ConfigLoader.get_instance(repo_root=self.repo_path)
			logger.debug("Created fallback ConfigLoader instance")
		else:
			self.config_loader = config_loader
			logger.debug("Using provided ConfigLoader instance")

		# Get RAG configuration
		rag_config = self.config_loader.get("rag", {})
		self.max_context_length = rag_config.get("max_context_length", DEFAULT_MAX_CONTEXT_LENGTH)
		self.max_context_results = rag_config.get("max_context_results", DEFAULT_MAX_CONTEXT_RESULTS)
		logger.debug(
			f"Using max_context_length: {self.max_context_length}, max_context_results: {self.max_context_results}"
		)

		self.db_client = DatabaseClient()  # Uses config path by default
		self.llm_client = create_client(
			repo_path=self.repo_path,
			model=model,  # create_client handles defaults/config
			api_key=api_key,
			api_base=api_base,
			config_loader=self.config_loader,  # Pass the config_loader down
		)
		# Initialize ProcessingPipeline correctly
		try:
			# Show a spinner while initializing the pipeline
			with progress_indicator(message="Initializing processing pipeline...", style="spinner", transient=True):
				# Use the provided ConfigLoader instance
				self.pipeline = ProcessingPipeline(
					repo_path=self.repo_path,  # Pipeline still needs repo_path directly
					config_loader=self.config_loader,  # Pass the correctly initialized loader
				)

			# Progress context manager handles completion message
			logger.info("ProcessingPipeline initialization complete.")
		except Exception:
			logger.exception("Failed to initialize ProcessingPipeline")
			self.pipeline = None

		logger.info(f"AskCommand initialized for session {self.session_id}")

	async def initialize(self) -> None:
		"""Perform asynchronous initialization for the command, especially the pipeline."""
		if self.pipeline and not self.pipeline.is_async_initialized:
			try:
				# Show a spinner while initializing the pipeline asynchronously
				with progress_indicator(
					message="Initializing async components (pipeline)...", style="spinner", transient=True
				):
					await self.pipeline.async_init(sync_on_init=True)
				logger.info("ProcessingPipeline async initialization complete.")
			except Exception:
				logger.exception("Failed during async initialization of ProcessingPipeline")
				# Optionally set pipeline to None or handle the error appropriately
				self.pipeline = None
		elif not self.pipeline:
			logger.error("Cannot perform async initialization: ProcessingPipeline failed to initialize earlier.")
		else:
			logger.info("AskCommand async components already initialized.")

	async def _retrieve_context(self, query: str, limit: int | None = None) -> list[dict[str, Any]]:
		"""Retrieve relevant code chunks based on the query."""
		if not self.pipeline:
			logger.warning("ProcessingPipeline not available, no context will be retrieved.")
			return []

		# Use configured limit or default
		actual_limit = limit or self.max_context_results

		try:
			logger.info(f"Retrieving context for query: '{query}', limit: {actual_limit}")
			# Use synchronous method to get results (pipeline.semantic_search is async)
			# Now call await directly as this method is async
			# import asyncio
			# results = asyncio.run(self.pipeline.semantic_search(query, k=actual_limit))
			results = await self.pipeline.semantic_search(query, k=actual_limit)

			# Format results for the LLM
			formatted_results = []
			if results:  # Check if results is not None and has items
				for r in results:
					# Extract relevant fields from payload
					payload = r.get("payload", {})

					# Get file content from repo using file_path, start_line, and end_line
					file_path = payload.get("file_path", "N/A")
					start_line = payload.get("start_line", -1)
					end_line = payload.get("end_line", -1)

					# Get content from repository if needed and build a content representation
					# For now, we'll use a simple representation that includes metadata
					entity_type = payload.get("entity_type", "")
					entity_name = payload.get("entity_name", "")
					language = payload.get("language", "")

					# Build a content representation from the metadata
					content_parts = []
					content_parts.append(f"Type: {entity_type}")
					if entity_name:
						content_parts.append(f"Name: {entity_name}")

					# Get the file content from the repo
					try:
						if self.repo_path and file_path and start_line > 0 and end_line > 0:
							repo_file_path = self.repo_path / file_path
							if repo_file_path.exists():
								with repo_file_path.open(encoding="utf-8") as f:
									file_content = f.read()
								lines = file_content.splitlines()
								if start_line <= len(lines) and end_line <= len(lines):
									code_content = "\n".join(lines[start_line - 1 : end_line])
									if language:
										content_parts.append(f"```{language}\n{code_content}\n```")
									else:
										content_parts.append(f"```\n{code_content}\n```")
					except Exception:
						logger.exception(f"Error reading file content for {file_path}")

					content = "\n\n".join(content_parts)

					formatted_results.append(
						{
							"file_path": file_path,
							"start_line": start_line,
							"end_line": end_line,
							"content": content,
							"score": r.get("score", -1.0),
						}
					)

			logger.debug(f"Semantic search returned {len(formatted_results)} results.")
			return formatted_results
		except Exception:
			logger.exception("Error retrieving context")
			return []

	def _extract_answer_from_response(self, response: Any) -> str | None:  # noqa: ANN401
		"""Safely extract the answer text from a litellm response object."""
		if response is None:
			return None

		try:
			# Handle different response object structures
			if hasattr(response, "choices") and response.choices:
				# Try to access as object with attributes
				choice = response.choices[0]
				if hasattr(choice, "message"):
					message = choice.message
					if hasattr(message, "content"):
						return message.content

			# Try to access as dictionary
			if isinstance(response, dict) and "choices" in response and response["choices"]:
				choice = response["choices"][0]
				if isinstance(choice, dict) and "message" in choice:
					message = choice["message"]
					if isinstance(message, dict) and "content" in message:
						return message["content"]

			# If we got here, we couldn't extract the answer
			logger.warning(f"Couldn't extract answer from response of type {type(response)}")
			return None

		# Fix for BLE001 (blind exception) - catch specific exceptions
		except (AttributeError, IndexError, KeyError, TypeError) as e:
			logger.warning(f"Error extracting answer from response: {e}")
			return None

	async def run(self, question: str) -> AskResult:
		"""Executes one turn of the ask command, returning the answer and context."""
		logger.info(f"Processing question for session {self.session_id}: '{question}'")

		# Ensure async initialization happened (idempotent check inside)
		await self.initialize()

		if not self.pipeline:
			return AskResult(answer="Processing pipeline not available.", context=[])

		# Retrieve relevant context first
		context = await self._retrieve_context(question)

		# Format context for inclusion in prompt
		context_text = format_content_for_context(context)
		if len(context_text) > self.max_context_length:
			logger.warning(f"Context too long ({len(context_text)} chars), truncating.")
			context_text = context_text[: self.max_context_length] + "... [truncated]"

		# Construct messages with context included
		messages = [
			{"role": "system", "content": SYSTEM_PROMPT},
			{
				"role": "user",
				"content": (
					f"Here's my question about the codebase: {question}\n\n"
					f"Relevant context from the codebase:\n{context_text}"
				),
			},
		]

		# Store user query in DB
		db_entry_id = None
		try:
			db_entry = self.db_client.add_chat_message(session_id=self.session_id, user_query=question)
			db_entry_id = db_entry.id if db_entry else None
			if db_entry_id:
				logger.debug(f"Stored current query turn with DB ID: {db_entry_id}")
			else:
				logger.warning("Failed to get DB entry ID for current query turn.")
		except Exception:
			logger.exception("Failed to store current query turn in DB")

		# Get LLM config from the injected ConfigLoader
		# At this point self.config_loader is guaranteed to be a valid instance
		llm_config = self.config_loader.get_llm_config()
		llm_params = {
			"temperature": llm_config.get("temperature", 0.7),
			"max_tokens": llm_config.get("max_tokens", 1024),
		}
		model_name = llm_config.get("model", "openai/gpt-4o-mini")
		api_base = llm_config.get("api_base")
		api_key_env_var = llm_config.get("api_key_env")
		api_key = None
		if isinstance(api_key_env_var, str):
			api_key = os.getenv(api_key_env_var)
		api_key = api_key or llm_config.get("api_key")  # Fallback to direct config key

		# Call LLM with context
		try:
			with loading_spinner("Waiting for LLM response..."):
				response = completion(
					model=model_name,
					messages=messages,
					api_base=api_base,
					api_key=api_key,
					stream=False,
					**llm_params,
				)

			# Extract answer from response
			answer = self._extract_answer_from_response(response)

			# Update DB with answer using the dedicated client method
			if db_entry_id and answer:
				# The update_chat_response method handles its own exceptions and returns success/failure
				success = self.db_client.update_chat_response(message_id=db_entry_id, ai_response=answer)
				if not success:
					logger.warning(f"Failed to update DB entry {db_entry_id} via client method.")

			return AskResult(answer=answer, context=context)
		except Exception as e:  # Keep the outer exception for LLM call errors
			logger.exception("Error during LLM completion")
			return AskResult(answer=f"Error: {e!s}", context=context)
